
#include <opencv2/opencv.hpp>

#include "infer.hpp"
#include "resnet.hpp"
#include "cpm.hpp"

using namespace std;


resnet::Image cvimg(const cv::Mat &image) { return resnet::Image(image.data, image.cols, image.rows); }

void perf() {
  int max_infer_batch = 1;
  int batch = 1;
  std::vector<cv::Mat> images{cv::imread("inference/car.jpg"), cv::imread("inference/car.jpg"),
                              cv::imread("inference/car.jpg")};

  for (int i = images.size(); i < batch; ++i) images.push_back(images[i % 3]);

  cpm::Instance<resnet::Attribute, resnet::Image, resnet::Infer> cpmi;
  bool ok = cpmi.start([] { return resnet::load("resnet.engine"); },
                       max_infer_batch);

  if (!ok) return;

  std::vector<resnet::Image> resnetimages(images.size());
  std::transform(images.begin(), images.end(), resnetimages.begin(), cvimg);

  trt::Timer timer;

  for (int i = 0; i < 100; ++i) {
    timer.start();
    cpmi.commit(resnetimages[0]).get();
    timer.stop("BATCH1");
  }
}

void single_inference() {
  cv::Mat image = cv::imread("inference/car.jpg");
  auto resnet = resnet::load("resnet.engine");
  if (resnet == nullptr) return;
  trt::Timer timer;
  for (int i = 0; i < 100; i++)
  {
    timer.start();
    auto attr = resnet->forward(cvimg(image));
    printf("score : %lf, label : %d\n", attr.confidence, attr.class_label);
    timer.stop("BATCH1");
  }
}

int main() {
  // [BATCH1]: 0.48650 ms
  single_inference();
  // [BATCH1]: 0.49974 ms
  perf();
  return 0;
}